#include <iostream>                 //Ein-/Ausgabe (Include-Dateien)
#include <math.h>                   //Mathematisches
#include <omp.h>                    //OpenMP
#include <stdio.h>                  //Fuer die Ausgabedatei

//Definition einer polytropen Zustandsgleichung
double eos(double p, double K, double gamma)
  {
    double e;
    e=pow(p/K,1.0/gamma);
    return e;
  }
  
//Definition einer MIT-Bag Zustandsgleichung mit cs^2=1/3 (Ueberladen der Funktion eos(...))
double eos(double p, double Bag)
  {
    double e;
    e=3.0*p + 4.0*Bag;
    return e;
  }  

main(void)  //Hauptprogramm
  {
    //Variablendeklarationen
    int i,anz=300;                                       //Pruefung8: Hier abgeaendert
    double M,p,e,r,nu,dM,dp,de,dr,dnu,dec;
    double Er[anz],EM[anz],Enu[anz],Eec[anz];
    double K,gamma;
    double Bag;
    
    //Ausgabedatei
    FILE *ausgabe;
    ausgabe = fopen("output/tov.txt", "w+");           //"tov.txt" im Unterverzeichnis "output" ffnen zum speichern der Ergebnisse
    
    //Variableninitialisierung
    dr=0.00001;
    dec=0.00005;
    gamma=5.0/3.0;
    K=4.2;                                                   //Pruefung8: Hier abgeaendert
    Bag=0.0001339578066;                                     //Entspricht B^(1/4)=170 MeV
    
    //for Schleife zur Berechnung mehrerer Sterne
#pragma omp parallel for private(i,M,p,e,r,nu,dM,dp,de,dnu)
    for (i=0;i<anz;i++)
    {
      M=0;
      r=pow(10,-14);
      p=K*pow((i+1)*dec,gamma);                          //Fuer polytrope Zustandsgleichung
//      p=1.0/3.0*(i+1)*dec-4.0/3.0*Bag;                   //Fuer MIT-Bag Zustandsgleichung
      nu=0;
      Eec[i]=eos(p,K,gamma);                             //Fuer polytrope Zustandsgleichung
//      Eec[i]=eos(p,Bag);                                 //Fuer MIT-Bag Zustandsgleichung
      
      //do-while Schleife (Numerische Lsung der TOV-Gleichung) 
      do
        {
          e=eos(p,K,gamma);                              //Wert der Energiedichte bei momentanen Druck (polytrope Zustandsgleichung)
//          e=eos(p,Bag);                                //Wert der Energiedichte bei momentanen Druck (MIT-Bag Zustandsgleichung)
          dM=4*M_PI*e*r*r*dr;                            //Massenzunahme bei momentanem r und Schrittweite dr
          dp=-(p+e)*(M+4*M_PI*r*r*r*p)/(r*(r-2*M))*dr;   //Druckzunahme bei momentanem r und Schrittweite dr (TOV-Gleichung)
          dnu=(M + 4*M_PI*r*r*r*p)/(r*(r-2*M))*dr;       //Metrikzunahme bei momentanem r und Schrittweite dr
          r=r+dr;                                        //momentaner Radius des Neutronensterns
          M=M+dM;                                        //momentane Masse des Neutronensterns innerhalb des Radius r
          p=p+dp;                                        //momentaner Druck des Neutronensterns innerhalb des Radius r
          nu=nu+dnu;                                     //momentane Metrik des Neutronensterns innerhalb des Radius r
        }
        while(p>0);
      
      Er[i]=r;
      EM[i]=M;
      Enu[i]=log(1-2*M/r)/2-nu; 
    }
    
    //Geordnete Ausgabe der Masse, des Radius und der zentralen g00-Metrikkomponente in die Ausgabedatei
    fprintf(ausgabe,"# R[km]    M[Msol]   g00    ec[MeV/fm3] \n"); 
    for (i=0;i<anz;i++)
    {
      fprintf(ausgabe, "%f %f %f %e \n",Er[i],EM[i]/1.4766,exp(2*Enu[i]),Eec[i]*pow(10,6)/1.3234); 
    }
    
    fclose(ausgabe);                                   //Ausgabedatei schliessen
    
    return 0;                                          //main beenden (Programmende)
  }

/*
Zusammenstellung einiger Zustandsgleichungen:

Neutronenstern: Einfaches nichtrelativistisches Fermigas Model ohne nukleare Wechselwirkungen 
(siehe z.B. I. Sagert, M. Hempel, C. Greiner, J. Schaffner-Bielich, "Compact Stars for Undergraduates", Eur.J.Phys.27:577-610,2006): 
    dr=0.00001;
    dec=0.00005;
    gamma=5.0/3.0;
    K=7.3015389;
        
Weisser Zwerg: Einfaches nichtrelativistisches Fermigas Model fuer Elektronen ohne Wechselwirkungen 
(siehe z.B. I. Sagert, M. Hempel, C. Greiner, J. Schaffner-Bielich, "Compact Stars for Undergraduates", Eur.J.Phys.27:577-610,2006):
Bemerkung: Bitte hier eine groessere Schrittweite und einen niedrigeren Wert fuer die zentrale Dichte benutzen
    dr=0.01;
    dec=pow(10,-14);
    gamma=5.0/3.0;
    K=3758.7193;
    
Neutronenstern: realistisches Model einer hadronischen Zustandsgleichung DD2
(siehe z.B.  S. Typel, G. Rpke, T. Klhn, D. Blaschke, H. H. Wolter, Phys. Rev. C 81 015803):
Bemerkung: Die hier benutzte polytrope Zustandsgleichung ist lediglich eine grobe Annherung an die wirkliche DD2-Zustandsgleichung (nicht gut fuer Dichten unterhalb normaler Kerndichte)
    dr=0.00001;
    dec=0.00005;
    gamma=2.5371;
    K=16902.4;
    
Quarksterne: MIT-Bag Zustandsgleichung mit cs^2=1/3 
(siehe z.B. M. G. Alford, S. Han, M. Prakash, Phys. Rev. D 88, 083013 (2013)):
Bemerkung: Sie muessen an drei Stellen im Code die MIT-Bag Zustandsgleichung auskommentieren und die polytrope mit // kommentieren
    dr=0.00001;
    dec=0.00005;    
    Bag=0.0001339578066;                                     //Entspricht B^(1/4)=170 MeV
*/

